function [iCross,iCrossRis,iCrossFal]=levelcross(y,level=0,yHyst=0,xDeadZone=0)

# [iCross, iCrossRis, iCrossFal] = levelcross(y, level, yHyst, xDeadZone)
#
# Determines vector indices at which y crosses the value of level and returns 
# them in the vector iCross. If level is not provided, it defaults to 0.
#
# The vectors iCrossRis and iCrossFal are subsets of iCross where indices 
# correspond to level crossings at rising or at falling edges of y, respectively.
# 
# If yHyst is given, then it is the height of the hysteresis band relative to 
# the peak-to-peak amplitude of y. A level crossing is then only detected where 
# y rises from below level-h/2 to above level+h/2 or in the opposite direction, 
# where
#
#     h = yHyst * (max(y) - min(y))
#
# If xDeadZone is given, then it is the length of the dead zone, i.e. the 
# minimum distance between two consecutive level crossings, relative to the 
# largest distance between two consecutive crossings detected. Any crossing 
# located closer than dth to the previous one is then ignored, where 
#
#     dth = xDeadZone * max(diff(cross))
#
# where cross is the vector of level crossing indices preliminarily detected.

# Author:
#   Lukasz Starzak 
#   Dept. of Microelectronics and Computer Science, Lodz University of Technology
  
  ySpan = max(y)-min(y);
  level *= ySpan;
  y = y - level;
  yHystHalf = ySpan*yHyst/2;
  iCross = [];
  isRisingEdge = [];
  
  iy = 1;
  while (y(iy) >= -yHystHalf && y(iy) <= yHystHalf)
    iy++;
  endwhile
  
  iyPrev = iy;
  for iy=[iyPrev+1:length(y)]
    if (y(iyPrev) < -yHystHalf && y(iy) > yHystHalf)
      iCross = [iCross iy];
      isRisingEdge = [isRisingEdge true];
      iyPrev = iy;
    elseif (y(iyPrev) > yHystHalf && y(iy) < -yHystHalf)
      iCross = [iCross iy];
      isRisingEdge = [isRisingEdge false];
      iyPrev = iy;
    endif
  endfor
  
  if (xDeadZone > 0)
    diffs = diff(iCross);
    diffThr = xDeadZone * max(diffs);
    diffsOver = (diffs >= diffThr);
    diffsOver = [diffsOver(1) diffsOver];
    iCross = iCross(diffsOver);
    isRisingEdge = isRisingEdge(diffsOver);
  endif

  iCrossRis = iCross(isRisingEdge == true);
  iCrossFal = iCross(isRisingEdge == false);
  
  if (0)
    figure(); hold on;
    plot(y+level);
    plot([1 length(y)], [level level], "-r");
    plot([1 length(y)], [level+yHystHalf level+yHystHalf], ":r");
    plot([1 length(y)], [level-yHystHalf level-yHystHalf], ":r");
    for ic=[1:length(iCrossRis)]
      plot([iCrossRis(ic) iCrossRis(ic)], [min(y) max(y)]+level, ":g");
    endfor
    for ic=[1:length(iCrossFal)]
      plot([iCrossFal(ic) iCrossFal(ic)], [min(y) max(y)]+level, ":m");
    endfor
  endif
  
endfunction
