function Spectr=spectr(y,t=[],ffund=0)
  
# Spectr = spectr(y, t, ffund)
#
# Uses fft() to calculate the frequency spectrum of a signal y and applies the 
# appropriate scaling. The results only cover the positive-frequency part of 
# the FFT spectrum. The number of the resulting frequency samples is 
# length(y)/2+1 for even length(y) or (length(y)+1)/2 for odd length(y). 
# Scaling is performed so that the total power of the resulting frequency 
# samples equals that of y.
#
# Time coordinates corresponding to the values found in y may be provided in a 
# separate vector t of equal size.
#
# The fundamental frequency of y can be passed as the optional parameter ffund. 
# Then, before calling fft(), y is cropped to the integer number of periods 
# (1/ffund) contained in y according to the timing defined by t.
#
# The Spectr structure contains the following elements which are all vectors 
# of the size indicated above, where index 1 corresponds to the direct 
# component, index 2 corresponds to the first alternating component etc.:
#   - k -- component numbers where 0 corresponds to the direct component, 
#       1 corresponds to the first alternating component etc.; note that k=1 
#       does not necessarily correspond to the fundamental frequency as defined 
#       by ffund, which effect can be achieved by calculating Spectr.f/ffund
#   - n -- the number of the highest-order frequency component, equivalent to 
#       max(Spectr.k)
#   - f -- frequency values corresponding to the consecutive components, based 
#       on the timing defined in t; if t is not provided, then Spectr.f=Spectr.k
#   - c -- complex values of the consecutive components
#   - m -- magnitudes of the consecutive components
#   - rms -- rms values of the consecutive components
#   - phi -- phase values of the consecutive components

# Author:
#   Lukasz Starzak 
#   Dept. of Microelectronics and Computer Science, Lodz University of Technology
  
  if ((!isempty(t)) && ffund > 0)
    [y,t,nPeriods] = cropperiods(y,t,ffund);
	endif
  
  l = length(y);  
  craw = fft(y);
  if (mod(l,2)==1)
    c = craw(1:(l+1)/2) * 2/l;
  else
    c = craw(1:(l/2+1)) * 2/l;
    c(l/2+1) = craw(l/2+1) * sqrt(2)/l;
  endif
  c(1) = craw(1)/l;
  
  Spectr.n = length(c)-1;
  Spectr.k = [0:Spectr.n];
  if (isempty(t))
    Spectr.f = Spectr.k;
  else
    Spectr.f = Spectr.k / sampledinterval(t);
  endif
  Spectr.c = c;
  Spectr.m = abs(c);
  Spectr.rms = Spectr.m / sqrt(2);
  Spectr.rms(1) = Spectr.m(1);
  Spectr.phi = arg(c);
  
endfunction
