function spectrplot(Spectr,yval="m",xvar="f",xmax=0,xmin=0,label=[])

# spectrplot(Spectr, yval, xaxis, xmax, xmin, labels, label)
#
# Plots the frequency spectrum of a signal as contained in the structure Spectr.
#
# The optional string yval specifies the kind of signal component value to be 
# plotted. It can contain one or more of the following characters:
#   - "m" (default) -- magnitude (absolute value)
#   - "r" -- rms value
#   - "p" -- phase (argument) in radians
#   - "d" -- phase (argument) in degrees
#   - "e" -- real part
#   - "i" -- imaginary part
#
# The optional string xvar specifies the X-axis variable. It can be one of the 
# following:
#   - "f" (default) -- frequency
#   - "k" -- component number (with 0 corresponding to the direct component, 
#       1 corresponding to the first alternating component etc.)
#
# The X-axis range can be specified with the optional parameters xmax and xmin. 
# If xmax = 0, then it is set automatically.
#
# If the optional parameter labels is provided, it should be a column vector 
# of strings specifying axis labels:
#   - label(1,:) is the Y-axis label
#   - label(2,:) is an optional X-axis label (if missing, xvar is used)
#
# For a description of Spectr, see spectr().

# Author:
#   Lukasz Starzak 
#   Dept. of Microelectronics and Computer Science, Lodz University of Technology

  nSubplots = length(yval);
  figure();
  
  switch (xvar)
    case "f"
      x = Spectr.f;
      xlbl = "f";
    case "k"
      x = Spectr.k;
      xlbl = "k";
    otherwise
      error("Incorrect X-axis variable.");
  endswitch
  if (size(label,1) > 1)
    xlbl = char(label(2,1));
  endif
  
  for iSubplot = [1:nSubplots]
    if (nSubplots > 1)
      subplot(nSubplots, 1, iSubplot);
    endif
    ylblsuf = "";
    switch (yval(iSubplot))
      case "m"
        y = Spectr.m;
        ylbl = "abs";
      case "r"
        y = Spectr.rms;
        ylbl = "rms";
      case "p"
        y = Spectr.phi;
        ylbl = "arg";
        ylblsuf = " / rad";
      case "d"
        y = rad2deg(Spectr.phi);
        ylbl = "arg";
        ylblsuf = " / deg";
      case "e"
        y = real(Spectr.c);
        ylbl = "real";
      case "i"
        y = imag(Spectr.c);
        ylbl = "imag";
      otherwise
        error("Incorrect signal value.");
    endswitch
    if (size(label,1) > 1)
      ylbl = [ylbl "(" char(label(1,1)) ")" ylblsuf];
    elseif (size(label,1) == 1)
      ylbl = [ylbl "(" char(label) ")" ylblsuf];
    endif

    stem(x, y);
    xlabel(xlbl);
    ylabel(ylbl);
    if (xmax > 0)
      xlim([xmin xmax]);
    elseif (xmin > 0)
      xlim([xmin max(x)]);
    endif
    grid on;
  endfor

endfunction
